package de.tu_darmstadt.tk.SmartHomeNetworkSim.core.anomalydetectors;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map.Entry;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Packet;
import weka.classifiers.trees.J48;
import weka.clusterers.EM;
import weka.clusterers.HierarchicalClusterer;
import weka.clusterers.SimpleKMeans;
import weka.core.EuclideanDistance;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.SelectedTag;

public class EMAnomalyDetector extends BaseAnomalyDetector {

	/**
	 * Classifier
	 */
	private EM clusterer;
	
	private int NUMBER_OF_CLUSTERS = 16;
	
	public EMAnomalyDetector() {
		super();
		clusterer = new EM();
		try {
			clusterer.setNumClusters(NUMBER_OF_CLUSTERS);
			clusterer.setMinStdDev(0.1);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * Try to classify the given packets and detect anomalies
	 * @param packets packets to be classified
	 */
	@Override
	public boolean classifyInstance(Instance instance) throws Exception {
		int x = clusterer.clusterInstance(instance);
		
		double[] dists = clusterer.distributionForInstance(instance);
		double dev = (1-dists[x])*1000;
		
		if(dev <= clusterer.getMinStdDev())	{
			System.out.println("EM dist: " + (dev));
			return true;
		}
		System.out.println("EM dist: " + (dev));
		return false;		
	}
	
	/**
	 * Train the model using the given instances
	 * @param instances training set, which should be learned
	 * 
	 */
	@Override
	public void trainModel(Instances instances) {
		try {
			clusterer.buildClusterer(instances);
		
			//save Model after clustering
			saveModel();
		} catch (Exception e) {
			System.out.println("EM failed");
			e.printStackTrace();
		}
		
		dataset = new Instances("Directions", atts, 100000);
	}
	

	@Override
	public void saveModel() throws Exception {
		weka.core.SerializationHelper.write("./data/models/em.model", clusterer);
	}
	
	public String getAlgoName() {
		return "EM";
	}
}
