package de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation;

import java.util.Collection;
import java.util.Iterator;
import java.util.Random;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.SettingsController;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Connection;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.NetworkManipulationAlgorithm;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Protocol;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.devices.FloatSensorDevice;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.distributionHandler.ConstantValueDistributionHandler;

public class NewDeviceAnomaly implements NetworkManipulationAlgorithm {
	
	@Override
	public void runAlgorithm(Controller controller, long currentTimeStep) {
		if(currentTimeStep == 2000) {
			SettingsController settingsController = controller.getSettingsController();
			Link link = controller.getNetworkController().getLinks().iterator().next();
			Connection con = controller.getNetworkController().getConnections().iterator().next();
			Protocol protocol = con.getProtocol();
			
			//Add new Device
			FloatSensorDevice newDevice = new FloatSensorDevice("New Device");	
			newDevice.setFSinfoName("temperature");
			newDevice.setFSval(24);
			newDevice.setX((int) settingsController.getWidth()*4);
			newDevice.setY((int) settingsController.getHeight()/4);
			newDevice.addLink(link);
			link.addDevice(newDevice);
	
			Port pNewDevice = new Port(newDevice, (short)80 , 900);
			pNewDevice.setTriggerHandler(new ConstantValueDistributionHandler(900));
			pNewDevice.setLastTrigger(0);
			pNewDevice.setJitter((short) (Math.random()*50));
			pNewDevice.setStatus(Port.SENDING);
			pNewDevice.setConnection(con);
			protocol.addDeviceOfRole(pNewDevice, 2);
			con.addSmartDevice(pNewDevice);
			newDevice.addPort(pNewDevice);
	
			controller.getNetworkController().addSmartDevice(newDevice);
			
			controller.notifyObservers();
		}
	}
}
